@@ -4,4 +4,4 @@ Ctypes wrapper module for IXXAT Virtual CAN Interface V3 on win32 systems
 Copyright (C) 2016 Giuseppe Corbelli <giuseppe.corbelli@weightpack.com>
 """
 
-from can.interfaces.ixxat.canlib import IXXATBus as Bus
+from can.interfaces.ixxat.canlib import IXXATBus
