@@ -4,8 +4,8 @@ Ctypes wrapper module for IXXAT Virtual CAN Interface V3 on win32 systems
 Copyright (C) 2016 Giuseppe Corbelli <giuseppe.corbelli@weightpack.com>
 """
 
-import binascii
 import ctypes
+import functools
 import logging
 import sys
 import time
@@ -13,45 +13,104 @@ import time
 from can import CanError, BusABC
 from can import Message
 from can.interfaces.ixxat import constants, structures
-
+from can.broadcastmanager import (LimitedDurationCyclicSendTaskABC,
+                                  RestartableCyclicTaskABC)
 from can.ctypesutil import CLibrary, HANDLE, PHANDLE
 
+from .constants import VCI_MAX_ERRSTRLEN
 from .exceptions import *
 
-__all__ = ["VCITimeout", "VCIError", "VCIDeviceNotFoundError", "IXXATBus"]
+__all__ = ["VCITimeout", "VCIError", "VCIDeviceNotFoundError", "IXXATBus", "vciFormatError"]
 
 log = logging.getLogger('can.ixxat')
 
-if (sys.version_info.major == 2):
-    _timer_function = time.clock
-elif (sys.version_info.major == 3):
+if ((sys.version_info.major == 3) and (sys.version_info.minor >= 3)):
     _timer_function = time.perf_counter
+else:
+    _timer_function = time.clock
+
+# Hack to have vciFormatError as a free function, see below
+vciFormatError = None
 
 # main ctypes instance
+_canlib = None
 if sys.platform == "win32":
     try:
         _canlib = CLibrary("vcinpl")
     except Exception as e:
         log.warning("Cannot load IXXAT vcinpl library: %s", e)
-        _canlib = None
 else:
     # Will not work on other systems, but have it importable anyway for
     # tests/sphinx
     log.warning("IXXAT VCI library does not work on %s platform", sys.platform)
-    _canlib = None
+
+
+def __vciFormatErrorExtended(library_instance, function, HRESULT, arguments):
+    """ Format a VCI error and attach failed function, decoded HRESULT and arguments
+        :param CLibrary library_instance:
+            Mapped instance of IXXAT vcinpl library
+        :param callable function:
+            Failed function
+        :param HRESULT HRESULT:
+            HRESULT returned by vcinpl call
+        :param arguments:
+            Arbitrary arguments tuple
+        :return:
+            Formatted string
+    """
+    #TODO: make sure we don't generate another exception
+    return "{} - arguments were {}".format(
+        __vciFormatError(library_instance, function, HRESULT),
+        arguments
+    )
+
+
+def __vciFormatError(library_instance, function, HRESULT):
+    """ Format a VCI error and attach failed function and decoded HRESULT
+        :param CLibrary library_instance:
+            Mapped instance of IXXAT vcinpl library
+        :param callable function:
+            Failed function
+        :param HRESULT HRESULT:
+            HRESULT returned by vcinpl call
+        :return:
+            Formatted string
+    """
+    buf = ctypes.create_string_buffer(VCI_MAX_ERRSTRLEN)
+    ctypes.memset(buf, 0, VCI_MAX_ERRSTRLEN)
+    library_instance.vciFormatError(HRESULT, buf, VCI_MAX_ERRSTRLEN)
+    return "function {} failed ({})".format(function._name, buf.value.decode('utf-8', 'replace'))
 
 
 def __check_status(result, function, arguments):
+    """
+    Check the result of a vcinpl function call and raise appropriate exception
+    in case of an error. Used as errcheck function when mapping C functions
+    with ctypes.
+        :param result:
+            Function call numeric result
+        :param callable function:
+            Called function
+        :param arguments:
+            Arbitrary arguments tuple
+        :raise:
+            :class:VCITimeout
+            :class:VCIRxQueueEmptyError
+            :class:StopIteration
+            :class:VCIError
+    """
     if isinstance(result, int):
         # Real return value is an unsigned long
         result = ctypes.c_ulong(result).value
 
-    if (result == constants.VCI_E_TIMEOUT):
+    if result == constants.VCI_E_TIMEOUT:
         raise VCITimeout("Function {} timed out".format(function._name))
-    elif (result == constants.VCI_E_NO_MORE_ITEMS):
+    elif result == constants.VCI_E_RXQUEUE_EMPTY:
+        raise VCIRxQueueEmptyError()
+    elif result == constants.VCI_E_NO_MORE_ITEMS:
         raise StopIteration()
-    elif (result != constants.VCI_OK):
-        raise VCIError(function, result, arguments)
+    elif result != constants.VCI_OK:
+        raise VCIError(vciFormatError(function, result))
 
     return result
 
@@ -62,6 +121,8 @@ try:
 
     #void VCIAPI vciFormatError (HRESULT hrError, PCHAR pszText, UINT32 dwsize);
     _canlib.map_symbol("vciFormatError", None, (ctypes.HRESULT, ctypes.c_char_p, ctypes.c_uint32))
+    # Hack to have vciFormatError as a free function
+    vciFormatError = functools.partial(__vciFormatError, _canlib)
 
     # HRESULT VCIAPI vciEnumDeviceOpen( OUT PHANDLE hEnum );
     _canlib.map_symbol("vciEnumDeviceOpen", ctypes.c_long, (PHANDLE,), __check_status)
@@ -93,6 +154,8 @@ try:
     _canlib.map_symbol("canChannelWaitRxEvent", ctypes.c_long, (HANDLE, ctypes.c_uint32), __check_status)
     #HRESULT canChannelPostMessage (HANDLE hChannel, PCANMSG pCanMsg );
     _canlib.map_symbol("canChannelPostMessage", ctypes.c_long, (HANDLE, structures.PCANMSG), __check_status)
+    #HRESULT canChannelSendMessage (HANDLE hChannel, UINT32 dwMsTimeout, PCANMSG pCanMsg );
+    _canlib.map_symbol("canChannelSendMessage", ctypes.c_long, (HANDLE, ctypes.c_uint32, structures.PCANMSG), __check_status)
 
     #EXTERN_C HRESULT VCIAPI canControlOpen( IN  HANDLE  hDevice, IN  UINT32  dwCanNo, OUT PHANDLE phCanCtl );
     _canlib.map_symbol("canControlOpen", ctypes.c_long, (HANDLE, ctypes.c_uint32, PHANDLE), __check_status)
@@ -114,6 +177,22 @@ try:
     _canlib.map_symbol("canControlAddFilterIds", ctypes.c_long, (HANDLE, ctypes.c_int, ctypes.c_uint32, ctypes.c_uint32), __check_status)
     #EXTERN_C HRESULT canControlRemFilterIds (HANDLE hControl, BOOL fExtendend, UINT32 dwCode, UINT32 dwMask );
     _canlib.map_symbol("canControlRemFilterIds", ctypes.c_long, (HANDLE, ctypes.c_int, ctypes.c_uint32, ctypes.c_uint32), __check_status)
+    #EXTERN_C HRESULT canSchedulerOpen (HANDLE hDevice, UINT32 dwCanNo, PHANDLE phScheduler );
+    _canlib.map_symbol("canSchedulerOpen", ctypes.c_long, (HANDLE, ctypes.c_uint32, PHANDLE), __check_status)
+    #EXTERN_C HRESULT canSchedulerClose (HANDLE hScheduler );
+    _canlib.map_symbol("canSchedulerClose", ctypes.c_long, (HANDLE, ), __check_status)
+    #EXTERN_C HRESULT canSchedulerGetCaps (HANDLE hScheduler, PCANCAPABILITIES pCaps );
+    _canlib.map_symbol("canSchedulerGetCaps", ctypes.c_long, (HANDLE, structures.PCANCAPABILITIES), __check_status)
+    #EXTERN_C HRESULT canSchedulerActivate ( HANDLE hScheduler, BOOL fEnable );
+    _canlib.map_symbol("canSchedulerActivate", ctypes.c_long, (HANDLE, ctypes.c_int), __check_status)
+    #EXTERN_C HRESULT canSchedulerAddMessage (HANDLE hScheduler, PCANCYCLICTXMSG pMessage, PUINT32 pdwIndex );
+    _canlib.map_symbol("canSchedulerAddMessage", ctypes.c_long, (HANDLE, structures.PCANCYCLICTXMSG, ctypes.POINTER(ctypes.c_uint32)), __check_status)
+    #EXTERN_C HRESULT canSchedulerRemMessage (HANDLE hScheduler, UINT32 dwIndex );
+    _canlib.map_symbol("canSchedulerRemMessage", ctypes.c_long, (HANDLE, ctypes.c_uint32), __check_status)
+    #EXTERN_C HRESULT canSchedulerStartMessage (HANDLE hScheduler, UINT32 dwIndex, UINT16 dwCount );
+    _canlib.map_symbol("canSchedulerStartMessage", ctypes.c_long, (HANDLE, ctypes.c_uint32, ctypes.c_uint16), __check_status)
+    #EXTERN_C HRESULT canSchedulerStopMessage (HANDLE hScheduler, UINT32 dwIndex );
+    _canlib.map_symbol("canSchedulerStopMessage", ctypes.c_long, (HANDLE, ctypes.c_uint32), __check_status)
     _canlib.vciInitialize()
 except AttributeError:
     # In case _canlib == None meaning we're not on win32/no lib found
@@ -126,7 +205,7 @@ except Exception as e:
 CAN_INFO_MESSAGES = {
     constants.CAN_INFO_START: "CAN started",
     constants.CAN_INFO_STOP: "CAN stopped",
-    constants.CAN_INFO_RESET:"CAN resetted",
+    constants.CAN_INFO_RESET: "CAN reset",
 }
 
 CAN_ERROR_MESSAGES = {
@@ -185,7 +264,7 @@ class IXXATBus(BusABC):
         :param int bitrate:
             Channel bitrate in bit/s
         """
-        if (_canlib is None):
+        if _canlib is None:
             raise ImportError("The IXXAT VCI library has not been initialized. Check the logs for more details.")
         log.info("CAN Filters: %s", can_filters)
         log.info("Got configuration of: %s", config)
@@ -194,7 +273,6 @@ class IXXATBus(BusABC):
         UniqueHardwareId = config.get('UniqueHardwareId', None)
         rxFifoSize = config.get('rxFifoSize', 16)
         txFifoSize = config.get('txFifoSize', 16)
-        extended = config.get('extended', False)
         # Usually comes as a string from the config file
         channel = int(channel)
 
@@ -210,14 +288,19 @@ class IXXATBus(BusABC):
         self._payload = (ctypes.c_byte * 8)()
 
         # Search for supplied device
-        log.info("Searching for unique HW ID %s", UniqueHardwareId)
+        if UniqueHardwareId is None:
+            log.info("Searching for first available device")
+        else:
+            log.info("Searching for unique HW ID %s", UniqueHardwareId)
         _canlib.vciEnumDeviceOpen(ctypes.byref(self._device_handle))
         while True:
             try:
                 _canlib.vciEnumDeviceNext(self._device_handle, ctypes.byref(self._device_info))
             except StopIteration:
-                # TODO: better error message
-                raise VCIDeviceNotFoundError("Unique HW ID {} not found".format(UniqueHardwareId))
+                if (UniqueHardwareId is None):
+                    raise VCIDeviceNotFoundError("No IXXAT device(s) connected or device(s) in use by other process(es).")
+                else:
+                    raise VCIDeviceNotFoundError("Unique HW ID {} not connected or not available.".format(UniqueHardwareId))
             else:
                 if (UniqueHardwareId is None) or (self._device_info.UniqueHardwareId.AsChar == bytes(UniqueHardwareId, 'ascii')):
                     break
@@ -235,12 +318,12 @@ class IXXATBus(BusABC):
         _canlib.canControlOpen(self._device_handle, channel, ctypes.byref(self._control_handle))
         _canlib.canControlInitialize(
             self._control_handle,
-            constants.CAN_OPMODE_STANDARD|constants.CAN_OPMODE_EXTENDED|constants.CAN_OPMODE_ERRFRAME if extended else constants.CAN_OPMODE_STANDARD|constants.CAN_OPMODE_ERRFRAME,
+            constants.CAN_OPMODE_STANDARD|constants.CAN_OPMODE_EXTENDED|constants.CAN_OPMODE_ERRFRAME,
             self.CHANNEL_BITRATES[0][bitrate],
             self.CHANNEL_BITRATES[1][bitrate]
         )
         _canlib.canControlGetCaps(self._control_handle, ctypes.byref(self._channel_capabilities))
-        
+
         # With receive messages, this field contains the relative reception time of
         # the message in ticks. The resolution of a tick can be calculated from the fields
         # dwClockFreq and dwTscDivisor of the structure  CANCAPABILITIES in accordance with the following formula:
@@ -252,27 +335,37 @@ class IXXATBus(BusABC):
         if can_filters is not None and len(can_filters):
             log.info("The IXXAT VCI backend is filtering messages")
             # Disable every message coming in
-            _canlib.canControlSetAccFilter(self._control_handle, 1 if extended else 0, constants.CAN_ACC_CODE_NONE, constants.CAN_ACC_MASK_NONE)
+            for extended in (0, 1):
+                _canlib.canControlSetAccFilter(self._control_handle,
+                                               extended,
+                                               constants.CAN_ACC_CODE_NONE,
+                                               constants.CAN_ACC_MASK_NONE)
             for can_filter in can_filters:
                 # Whitelist
                 code = int(can_filter['can_id'])
                 mask = int(can_filter['can_mask'])
-                _canlib.canControlAddFilterIds(self._control_handle, 1 if extended else 0, code, mask)
-                rtr = (code & 0x01) and (mask & 0x01)
-                log.info("Accepting ID:%d  MASK:%d RTR:%s", code>>1, mask>>1, "YES" if rtr else "NO")
+                extended = can_filter.get('extended', False)
+                _canlib.canControlAddFilterIds(self._control_handle,
+                                               1 if extended else 0,
+                                               code << 1,
+                                               mask << 1)
+                log.info("Accepting ID: 0x%X MASK: 0x%X", code, mask)
 
         # Start the CAN controller. Messages will be forwarded to the channel
         _canlib.canControlStart(self._control_handle, constants.TRUE)
 
+        # For cyclic transmit list. Set when .send_periodic() is first called
+        self._scheduler = None
+        self._scheduler_resolution = None
+        self.channel = channel
+
         # Usually you get back 3 messages like "CAN initialized" ecc...
-        # Filter them out with low timeout
-        while (True):
+        # Clear the FIFO by filter them out with low timeout
+        for i in range(rxFifoSize):
             try:
-                _canlib.canChannelWaitRxEvent(self._channel_handle, 0)
-            except VCITimeout:
-                break
-            else:
                 _canlib.canChannelReadMessage(self._channel_handle, 0, ctypes.byref(self._message))
+            except (VCITimeout, VCIRxQueueEmptyError):
+                break
 
         super(IXXATBus, self).__init__()
 
@@ -285,69 +378,64 @@ class IXXATBus(BusABC):
             return 1
 
     def flush_tx_buffer(self):
-        " Flushes the transmit buffer on the IXXAT "
-        # TODO: no timeout?
+        """ Flushes the transmit buffer on the IXXAT """
+        # TODO #64: no timeout?
         _canlib.canChannelWaitTxEvent(self._channel_handle, constants.INFINITE)
 
     def recv(self, timeout=None):
-        " Read a message from IXXAT device. "
+        """ Read a message from IXXAT device. """
 
         # TODO: handling CAN error messages?
-        if (timeout is None):
-            timeout = constants.INFINITE
+        data_received = False
 
-        tm = None
-        if (timeout == 0):
+        if timeout == 0:
             # Peek without waiting
             try:
                 _canlib.canChannelPeekMessage(self._channel_handle, ctypes.byref(self._message))
-            except VCITimeout:
+            except (VCITimeout, VCIRxQueueEmptyError):
                 return None
-            except VCIError as e:
-                if (e.HRESULT == constants.VCI_E_RXQUEUE_EMPTY):
-                    return None
-                else:
-                    raise e
             else:
-                if (self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_DATA):
-                    tm = _timer_function()
+                if self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_DATA:
+                    data_received = True
         else:
             # Wait if no message available
-            t0 = _timer_function()
-            elapsed_ms = 0
-            remaining_ms = 0
-            while (elapsed_ms <= timeout):
-                elapsed_ms = int((_timer_function() - t0) * 1000)
-                remaining_ms = timeout - elapsed_ms
-                # Wait until at least one frame is in the buffer
+            if timeout is None or timeout < 0:
+                remaining_ms = constants.INFINITE
+                t0 = None
+            else:
+                timeout_ms = int(timeout * 1000)
+                remaining_ms = timeout_ms
+                t0 = _timer_function()
+
+            while True:
                 try:
-                    _canlib.canChannelWaitRxEvent(self._channel_handle, remaining_ms)
-                except VCITimeout:
-                    log.debug('canChannelWaitRxEvent timed out after %dms', remaining_ms)
-                    return None
-
-                # In theory we should be fine with a 0 timeout since the rxEvent was already
-                # set but I've seen timeouts appearing here and there
-                try:
-                    _canlib.canChannelReadMessage(self._channel_handle, 0, ctypes.byref(self._message))
-                except VCITimeout:
-                    continue
-
-                # See if we got a data or info/error messages
-                if (self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_DATA):
-                    tm = _timer_function()
-                    break
-
-                elif (self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_INFO):
-                    log.info(CAN_INFO_MESSAGES.get(self._message.abData[0], "Unknown CAN info message code {}".format(self._message.abData[0])))
-
-                elif (self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_ERROR):
-                    log.warning(CAN_ERROR_MESSAGES.get(self._message.abData[0], "Unknown CAN error message code {}".format(self._message.abData[0])))
-
-                elif (self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_TIMEOVR):
+                    _canlib.canChannelReadMessage(self._channel_handle, remaining_ms, ctypes.byref(self._message))
+                except (VCITimeout, VCIRxQueueEmptyError):
+                    # Ignore the 2 errors, the timeout is handled manually with the _timer_function()
                     pass
+                else:
+                    # See if we got a data or info/error messages
+                    if self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_DATA:
+                        data_received = True
+                        break
 
-        if (not tm):
+                    elif self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_INFO:
+                        log.info(CAN_INFO_MESSAGES.get(self._message.abData[0], "Unknown CAN info message code {}".format(self._message.abData[0])))
+
+                    elif self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_ERROR:
+                        log.warning(CAN_ERROR_MESSAGES.get(self._message.abData[0], "Unknown CAN error message code {}".format(self._message.abData[0])))
+
+                    elif self._message.uMsgInfo.Bits.type == constants.CAN_MSGTYPE_TIMEOVR:
+                        pass
+                    else:
+                        log.warn("Unexpected message info type")
+
+                if t0 is not None:
+                    remaining_ms = timeout_ms - int((_timer_function() - t0) * 1000)
+                    if remaining_ms < 0:
+                        break
+
+        if not data_received:
             # Timed out / can message type is not DATA
             return None
 
@@ -360,32 +448,94 @@ class IXXATBus(BusABC):
             False,
             self._message.dwMsgId,
             self._message.uMsgInfo.Bits.dlc,
-            self._message.abData
+            self._message.abData[:self._message.uMsgInfo.Bits.dlc],
+            self.channel
         )
 
         log.debug('Recv()ed message %s', rx_msg)
         return rx_msg
 
-    def send(self, msg):
+    def send(self, msg, timeout=None):
         log.debug("Sending message: %s", msg)
 
         # This system is not designed to be very efficient
-        ctypes.memset(ctypes.byref(self._message), 0, ctypes.sizeof(structures.CANMSG))
-        self._message.uMsgInfo.Bits.type = constants.CAN_MSGTYPE_DATA
-        self._message.uMsgInfo.Bits.rtr = 1 if msg.is_remote_frame else 0
-        self._message.uMsgInfo.Bits.ext = 1 if msg.id_type else 0
-        self._message.dwMsgId = msg.arbitration_id
-        if (msg.dlc):
-            self._message.uMsgInfo.Bits.dlc = msg.dlc
-            adapter = (ctypes.c_uint8 * msg.dlc).from_buffer(msg.data)
-            ctypes.memmove(self._message.abData, adapter, msg.dlc)
+        message = structures.CANMSG()
+        message.uMsgInfo.Bits.type = constants.CAN_MSGTYPE_DATA
+        message.uMsgInfo.Bits.rtr = 1 if msg.is_remote_frame else 0
+        message.uMsgInfo.Bits.ext = 1 if msg.id_type else 0
+        message.dwMsgId = msg.arbitration_id
+        if msg.dlc:
+            message.uMsgInfo.Bits.dlc = msg.dlc
+            adapter = (ctypes.c_uint8 * len(msg.data)).from_buffer(msg.data)
+            ctypes.memmove(message.abData, adapter, len(msg.data))
 
-        # This does not block but may raise if TX fifo is full
-        # if you prefer a blocking call use canChannelSendMessage
-        _canlib.canChannelPostMessage (self._channel_handle, self._message)
+        if timeout:
+            _canlib.canChannelSendMessage(
+                self._channel_handle, int(timeout * 1000), message)
+        else:
+            _canlib.canChannelPostMessage(self._channel_handle, message)
+
+    def send_periodic(self, msg, period, duration=None):
+        """Send a message using built-in cyclic transmit list functionality."""
+        if self._scheduler is None:
+            self._scheduler = HANDLE()
+            _canlib.canSchedulerOpen(self._device_handle, self.channel,
+                                     self._scheduler)
+            caps = structures.CANCAPABILITIES()
+            _canlib.canSchedulerGetCaps(self._scheduler, caps)
+            self._scheduler_resolution = float(caps.dwClockFreq) / caps.dwCmsDivisor
+            _canlib.canSchedulerActivate(self._scheduler, constants.TRUE)
+        return CyclicSendTask(self._scheduler, msg, period, duration,
+                              self._scheduler_resolution)
 
     def shutdown(self):
+        if self._scheduler is not None:
+            _canlib.canSchedulerClose(self._scheduler)
         _canlib.canChannelClose(self._channel_handle)
         _canlib.canControlStart(self._control_handle, constants.FALSE)
         _canlib.canControlClose(self._control_handle)
         _canlib.vciDeviceClose(self._device_handle)
+
+
+class CyclicSendTask(LimitedDurationCyclicSendTaskABC,
+                     RestartableCyclicTaskABC):
+    """A message in the cyclic transmit list."""
+
+    def __init__(self, scheduler, msg, period, duration, resolution):
+        super(CyclicSendTask, self).__init__(msg, period, duration)
+        self._scheduler = scheduler
+        self._index = None
+        self._count = int(duration / period) if duration else 0
+
+        self._msg = structures.CANCYCLICTXMSG()
+        self._msg.wCycleTime = int(round(period * resolution))
+        self._msg.dwMsgId = msg.arbitration_id
+        self._msg.uMsgInfo.Bits.type = constants.CAN_MSGTYPE_DATA
+        self._msg.uMsgInfo.Bits.ext = 1 if msg.id_type else 0
+        self._msg.uMsgInfo.Bits.rtr = 1 if msg.is_remote_frame else 0
+        self._msg.uMsgInfo.Bits.dlc = msg.dlc
+        for i, b in enumerate(msg.data):
+            self._msg.abData[i] = b
+        self.start()
+
+    def start(self):
+        """Start transmitting message (add to list if needed)."""
+        if self._index is None:
+            self._index = ctypes.c_uint32()
+            _canlib.canSchedulerAddMessage(self._scheduler,
+                                           self._msg,
+                                           self._index)
+        _canlib.canSchedulerStartMessage(self._scheduler,
+                                         self._index,
+                                         self._count)
+
+    def pause(self):
+        """Pause transmitting message (keep it in the list)."""
+        _canlib.canSchedulerStopMessage(self._scheduler, self._index)
+
+    def stop(self):
+        """Stop transmitting message (remove from list)."""
+        # Remove it completely instead of just stopping it to avoid filling up
+        # the list with permanently stopped messages
+        _canlib.canSchedulerRemMessage(self._scheduler, self._index)
+        self._index = None
