@@ -13,6 +13,7 @@ class LUID(ctypes.Structure):
     ]
 PLUID = ctypes.POINTER(LUID)
 
+
 class VCIID(ctypes.Union):
     _fields_ = [
         ("AsLuid", LUID),
@@ -20,6 +21,7 @@ class VCIID(ctypes.Union):
     ]
 PVCIID = ctypes.POINTER(VCIID)
 
+
 class GUID(ctypes.Structure):
     _fields_ = [
         ("Data1", ctypes.c_long),
@@ -28,6 +30,7 @@ class GUID(ctypes.Structure):
         ("Data4", ctypes.c_char * 8),
     ]
 
+
 class VCIDEVICEINFO(ctypes.Structure):
     class UniqueHardwareId(ctypes.Union):
         _fields_ = [
@@ -49,6 +52,7 @@ class VCIDEVICEINFO(ctypes.Structure):
         ("Manufacturer", ctypes.c_char * 126),
         ("DriverReleaseVersion", ctypes.c_uint16),
     ]
+
     def __str__(self):
         return "Mfg: {}, Dev: {} HW: {}.{}.{}.{} Drv: {}.{}.{}.{}".format(
             self.Manufacturer,
@@ -64,6 +68,7 @@ class VCIDEVICEINFO(ctypes.Structure):
         )
 PVCIDEVICEINFO = ctypes.POINTER(VCIDEVICEINFO)
 
+
 class CANLINESTATUS(ctypes.Structure):
     _fields_ = [
         ("bOpMode", ctypes.c_uint8),
@@ -85,6 +90,7 @@ class CANCHANSTATUS(ctypes.Structure):
     ]
 PCANCHANSTATUS = ctypes.POINTER(CANCHANSTATUS)
 
+
 class CANCAPABILITIES(ctypes.Structure):
     _fields_ = [
         ("wCtrlType", ctypes.c_uint16),
@@ -99,6 +105,7 @@ class CANCAPABILITIES(ctypes.Structure):
     ]
 PCANCAPABILITIES = ctypes.POINTER(CANCAPABILITIES)
 
+
 class CANMSGINFO(ctypes.Union):
     class Bytes(ctypes.Structure):
         _fields_ = [
@@ -128,6 +135,7 @@ class CANMSGINFO(ctypes.Union):
     ]
 PCANMSGINFO = ctypes.POINTER(CANMSGINFO)
 
+
 class CANMSG(ctypes.Structure):
     _fields_ = [
         ("dwTime", ctypes.c_uint32),
@@ -136,3 +144,14 @@ class CANMSG(ctypes.Structure):
         ("abData", ctypes.c_uint8 * 8)
     ]
 PCANMSG = ctypes.POINTER(CANMSG)
+
+class CANCYCLICTXMSG(ctypes.Structure):
+    _fields_ = [
+        ("wCycleTime", ctypes.c_uint16),
+        ("bIncrMode", ctypes.c_uint8),
+        ("bByteIndex", ctypes.c_uint8),
+        ("dwMsgId", ctypes.c_uint32),
+        ("uMsgInfo", CANMSGINFO),
+        ("abData", ctypes.c_uint8 * 8)
+    ]
+PCANCYCLICTXMSG = ctypes.POINTER(CANCYCLICTXMSG)
