@@ -12,17 +12,17 @@ import time
 import logging
 import ctypes
 
-log = logging.getLogger('can.canlib')
-log.setLevel(logging.INFO)
-
 from can import CanError, BusABC
 from can import Message
 from can.interfaces.kvaser import constants as canstat
 
+log = logging.getLogger('can.kvaser')
 
 # Resolution in us
 TIMESTAMP_RESOLUTION = 10
 
+TIMESTAMP_FACTOR = TIMESTAMP_RESOLUTION / 1000000.0
+
 
 try:
     if sys.platform == "win32":
@@ -40,16 +40,16 @@ def _unimplemented_function(*args):
 
 
 def __get_canlib_function(func_name, argtypes=[], restype=None, errcheck=None):
-    log.debug('Wrapping function "%s"' % func_name)
+    #log.debug('Wrapping function "%s"' % func_name)
     try:
         # e.g. canlib.canBusOn
         retval = getattr(__canlib, func_name)
-        log.debug('"%s" found in library', func_name)
+        #log.debug('"%s" found in library', func_name)
     except AttributeError:
         log.warning('"%s" was not found in library', func_name)
         return _unimplemented_function
     else:
-        log.debug('Result type is: %s' % type(restype))
+        #log.debug('Result type is: %s' % type(restype))
         #log.debug('Error check function is: %s' % errcheck)
         retval.argtypes = argtypes
         retval.restype = restype
@@ -81,7 +81,7 @@ class CANLIBError(CanError):
 
 
 def __convert_can_status_to_int(result):
-    log.debug("converting can status to int {} ({})".format(result, type(result)))
+    #log.debug("converting can status to int {} ({})".format(result, type(result)))
     if isinstance(result, int):
         return result
     else:
@@ -91,7 +91,7 @@ def __convert_can_status_to_int(result):
 def __check_status(result, function, arguments):
     result = __convert_can_status_to_int(result)
     if not canstat.CANSTATUS_SUCCESS(result):
-        log.debug('Detected error while checking CAN status')
+        #log.debug('Detected error while checking CAN status')
         raise CANLIBError(function, result, arguments)
     return result
 
@@ -99,7 +99,7 @@ def __check_status(result, function, arguments):
 def __check_status_read(result, function, arguments):
     result = __convert_can_status_to_int(result)
     if not canstat.CANSTATUS_SUCCESS(result) and result != canstat.canERR_NOMSG:
-        log.debug('Detected error in which checking status read')
+        #log.debug('Detected error in which checking status read')
         raise CANLIBError(function, result, arguments)
     return result
 
@@ -121,127 +121,117 @@ def __check_bus_handle_validity(handle, function, arguments):
     else:
         return handle
 
+if __canlib is not None:
+    canInitializeLibrary = __get_canlib_function("canInitializeLibrary")
 
-canInitializeLibrary = __get_canlib_function("canInitializeLibrary")
+    canGetErrorText = __get_canlib_function("canGetErrorText",
+                                            argtypes=[canstat.c_canStatus, ctypes.c_char_p, ctypes.c_uint],
+                                            restype=canstat.c_canStatus,
+                                            errcheck=__check_status)
 
-canGetErrorText = __get_canlib_function("canGetErrorText",
-                                        argtypes=[canstat.c_canStatus, ctypes.c_char_p, ctypes.c_uint],
+    # TODO wrap this type of function to provide a more Pythonic API
+    canGetNumberOfChannels = __get_canlib_function("canGetNumberOfChannels",
+                                                   argtypes=[ctypes.c_void_p],
+                                                   restype=canstat.c_canStatus,
+                                                   errcheck=__check_status)
+
+    kvReadTimer = __get_canlib_function("kvReadTimer",
+                                        argtypes=[c_canHandle,
+                                                  ctypes.POINTER(ctypes.c_uint)],
                                         restype=canstat.c_canStatus,
                                         errcheck=__check_status)
 
-# TODO wrap this type of function to provide a more Pythonic API
-canGetNumberOfChannels = __get_canlib_function("canGetNumberOfChannels",
-                                               argtypes=[ctypes.c_void_p],
-                                               restype=canstat.c_canStatus,
-                                               errcheck=__check_status)
+    canBusOff = __get_canlib_function("canBusOff",
+                                      argtypes=[c_canHandle],
+                                      restype=canstat.c_canStatus,
+                                      errcheck=__check_status)
 
-if sys.platform == "win32":
-    __canReadTimer_func_name = "kvReadTimer"
-else:
-    __canReadTimer_func_name = "canReadTimer"
-canReadTimer = __get_canlib_function(__canReadTimer_func_name,
-                                     argtypes=[c_canHandle, ctypes.c_void_p],
+    canBusOn = __get_canlib_function("canBusOn",
+                                     argtypes=[c_canHandle],
                                      restype=canstat.c_canStatus,
                                      errcheck=__check_status)
 
-canBusOff = __get_canlib_function("canBusOff",
-                                  argtypes=[c_canHandle],
-                                  restype=canstat.c_canStatus,
-                                  errcheck=__check_status)
+    canClose = __get_canlib_function("canClose",
+                                     argtypes=[c_canHandle],
+                                     restype=canstat.c_canStatus,
+                                     errcheck=__check_status)
 
-canBusOn = __get_canlib_function("canBusOn",
-                                 argtypes=[c_canHandle],
-                                 restype=canstat.c_canStatus,
-                                 errcheck=__check_status)
+    canOpenChannel = __get_canlib_function("canOpenChannel",
+                                           argtypes=[ctypes.c_int, ctypes.c_int],
+                                           restype=c_canHandle,
+                                           errcheck=__check_bus_handle_validity)
 
-canClose = __get_canlib_function("canClose",
-                                 argtypes=[c_canHandle],
-                                 restype=canstat.c_canStatus,
-                                 errcheck=__check_status)
+    canSetBusParams = __get_canlib_function("canSetBusParams",
+                                            argtypes=[c_canHandle, ctypes.c_long,
+                                                      ctypes.c_uint, ctypes.c_uint,
+                                                      ctypes.c_uint, ctypes.c_uint,
+                                                      ctypes.c_uint],
+                                            restype=canstat.c_canStatus,
+                                            errcheck=__check_status)
 
-canOpenChannel = __get_canlib_function("canOpenChannel",
-                                       argtypes=[ctypes.c_int, ctypes.c_int],
-                                       restype=c_canHandle,
-                                       errcheck=__check_bus_handle_validity)
 
-canSetBusParams = __get_canlib_function("canSetBusParams",
-                                        argtypes=[c_canHandle, ctypes.c_long,
-                                                  ctypes.c_uint, ctypes.c_uint,
-                                                  ctypes.c_uint, ctypes.c_uint,
-                                                  ctypes.c_uint],
+    canSetBusOutputControl = __get_canlib_function("canSetBusOutputControl",
+                                                   argtypes=[c_canHandle,
+                                                             ctypes.c_uint],
+                                                   restype=canstat.c_canStatus,
+                                                   errcheck=__check_status)
+
+    canSetAcceptanceFilter = __get_canlib_function("canSetAcceptanceFilter",
+                                                   argtypes=[
+                                                       c_canHandle,
+                                                       ctypes.c_uint,
+                                                       ctypes.c_uint,
+                                                       ctypes.c_int
+                                                   ],
+                                                   restype=canstat.c_canStatus,
+                                                   errcheck=__check_status)
+
+    canReadWait = __get_canlib_function("canReadWait",
+                                        argtypes=[c_canHandle, ctypes.c_void_p,
+                                                  ctypes.c_void_p, ctypes.c_void_p,
+                                                  ctypes.c_void_p, ctypes.c_void_p,
+                                                  ctypes.c_long],
                                         restype=canstat.c_canStatus,
-                                        errcheck=__check_status)
+                                        errcheck=__check_status_read)
 
-
-canSetBusOutputControl = __get_canlib_function("canSetBusOutputControl",
-                                               argtypes=[c_canHandle,
-                                                         ctypes.c_uint],
-                                               restype=canstat.c_canStatus,
-                                               errcheck=__check_status)
-
-canSetAcceptanceFilter = __get_canlib_function("canSetAcceptanceFilter",
-                                               argtypes=[
-                                                   c_canHandle,
-                                                   ctypes.c_uint,
-                                                   ctypes.c_uint,
-                                                   ctypes.c_int
-                                               ],
-                                               restype=canstat.c_canStatus,
-                                               errcheck=__check_status)
-
-canReadWait = __get_canlib_function("canReadWait",
-                                    argtypes=[c_canHandle, ctypes.c_void_p,
-                                              ctypes.c_void_p, ctypes.c_void_p,
-                                              ctypes.c_void_p, ctypes.c_void_p,
-                                              ctypes.c_long],
-                                    restype=canstat.c_canStatus,
-                                    errcheck=__check_status_read)
-
-canWriteWait = __get_canlib_function("canWriteWait",
+    canWrite = __get_canlib_function("canWrite",
                                      argtypes=[
                                          c_canHandle,
                                          ctypes.c_long,
                                          ctypes.c_void_p,
                                          ctypes.c_uint,
-                                         ctypes.c_uint,
-                                         ctypes.c_ulong],
+                                         ctypes.c_uint],
                                      restype=canstat.c_canStatus,
                                      errcheck=__check_status)
 
+    canWriteSync = __get_canlib_function("canWriteSync",
+                                         argtypes=[c_canHandle, ctypes.c_ulong],
+                                         restype=canstat.c_canStatus,
+                                         errcheck=__check_status)
 
-canIoCtl = __get_canlib_function("canIoCtl",
-                                 argtypes=[c_canHandle, ctypes.c_uint,
-                                           ctypes.c_void_p, ctypes.c_uint],
-                                 restype=canstat.c_canStatus,
-                                 errcheck=__check_status)
+    canIoCtl = __get_canlib_function("canIoCtl",
+                                     argtypes=[c_canHandle, ctypes.c_uint,
+                                               ctypes.c_void_p, ctypes.c_uint],
+                                     restype=canstat.c_canStatus,
+                                     errcheck=__check_status)
 
-canGetVersion = __get_canlib_function("canGetVersion",
-                                      restype=ctypes.c_short,
-                                      errcheck=__check_status)
+    canGetVersion = __get_canlib_function("canGetVersion",
+                                          restype=ctypes.c_short,
+                                          errcheck=__check_status)
 
-kvFlashLeds = __get_canlib_function("kvFlashLeds",
-                                    argtypes=[c_canHandle, ctypes.c_int,
-                                              ctypes.c_int],
-                                    restype=ctypes.c_short,
-                                    errcheck=__check_status)
+    kvFlashLeds = __get_canlib_function("kvFlashLeds",
+                                        argtypes=[c_canHandle, ctypes.c_int,
+                                                  ctypes.c_int],
+                                        restype=ctypes.c_short,
+                                        errcheck=__check_status)
 
-if sys.platform == "win32":
-    canGetVersionEx = __get_canlib_function("canGetVersionEx",
-                                            argtypes=[ctypes.c_uint],
-                                            restype=ctypes.c_uint,
-                                            errcheck=__check_status)
+    if sys.platform == "win32":
+        canGetVersionEx = __get_canlib_function("canGetVersionEx",
+                                                argtypes=[ctypes.c_uint],
+                                                restype=ctypes.c_uint,
+                                                errcheck=__check_status)
 
-
-def init_kvaser_library():
-    try:
-        log.debug("Initializing Kvaser CAN library")
-        canInitializeLibrary()
-        log.debug("CAN library initialized")
-    except:
-        log.warning("Kvaser canlib could not be initialized.")
-
-
-canGetChannelData = __get_canlib_function("canGetChannelData",
+    canGetChannelData = __get_canlib_function("canGetChannelData",
                                           argtypes=[ctypes.c_int,
                                                     ctypes.c_int,
                                                     ctypes.c_void_p,
@@ -250,6 +240,16 @@ canGetChannelData = __get_canlib_function("canGetChannelData",
                                           errcheck=__check_status)
 
 
+def init_kvaser_library():
+    if __canlib is not None:
+        try:
+            log.debug("Initializing Kvaser CAN library")
+            canInitializeLibrary()
+            log.debug("CAN library initialized")
+        except:
+            log.warning("Kvaser canlib could not be initialized.")
+
+
 DRIVER_MODE_SILENT = False
 DRIVER_MODE_NORMAL = True
 
@@ -308,6 +308,11 @@ class KvaserBus(BusABC):
         :param bool single_handle:
             Use one Kvaser CANLIB bus handle for both reading and writing.
             This can be set if reading and/or writing is done from one thread.
+        :param bool receive_own_messages:
+            If messages transmitted should also be received back.
+            Only works if single_handle is also False.
+            If you want to receive messages from other applications on the same
+            computer, set this to True or set single_handle to True.
         """
         log.info("CAN Filters: {}".format(can_filters))
         log.info("Got configuration of: {}".format(config))
@@ -318,11 +323,13 @@ class KvaserBus(BusABC):
         no_samp = config.get('no_samp', 0)
         driver_mode = config.get('driver_mode', DRIVER_MODE_NORMAL)
         single_handle = config.get('single_handle', False)
+        receive_own_messages = config.get('receive_own_messages', False)
 
         try:
             channel = int(channel)
         except ValueError:
             raise ValueError('channel must be an integer')
+        self.channel = channel
 
         if 'tseg1' not in config and bitrate in BITRATE_OBJS:
             bitrate = BITRATE_OBJS[bitrate]
@@ -332,7 +339,7 @@ class KvaserBus(BusABC):
 
         num_channels = ctypes.c_int(0)
         res = canGetNumberOfChannels(ctypes.byref(num_channels))
-        log.debug("Res: {}".format(res))
+        #log.debug("Res: {}".format(res))
         num_channels = int(num_channels.value)
         log.info('Found %d available channels' % num_channels)
         for idx in range(num_channels):
@@ -349,6 +356,15 @@ class KvaserBus(BusABC):
                  4)
         canSetBusParams(self._read_handle, bitrate, tseg1, tseg2, sjw, no_samp, 0)
 
+        # By default, use local echo if single handle is used (see #160)
+        local_echo = single_handle or receive_own_messages
+        if receive_own_messages and single_handle:
+            log.warning("receive_own_messages only works if single_handle is False")
+        canIoCtl(self._read_handle,
+                 canstat.canIOCTL_SET_LOCAL_TXECHO,
+                 ctypes.byref(ctypes.c_byte(local_echo)),
+                 1)
+
         if self.single_handle:
             log.debug("We don't require separate handles to the bus")
             self._write_handle = self._read_handle
@@ -357,7 +373,6 @@ class KvaserBus(BusABC):
             self._write_handle = canOpenChannel(channel, canstat.canOPEN_ACCEPT_VIRTUAL)
             canBusOn(self._read_handle)
 
-        self.sw_filters = []
         self.set_filters(can_filters)
 
         can_driver_mode = canstat.canDRIVER_SILENT if driver_mode == DRIVER_MODE_SILENT else canstat.canDRIVER_NORMAL
@@ -365,15 +380,13 @@ class KvaserBus(BusABC):
         log.debug('Going bus on TX handle')
         canBusOn(self._write_handle)
 
-        self.timer_offset = None  # Used to zero the timestamps from the first message
-
-        '''
-        Approximate offset between time.time() and CAN timestamps (~2ms accuracy)
-        There will always be some lag between when the message is on the bus to
-        when it reaches Python. Allow messages to be on the bus for a while before
-        reading this value so it has a chance to correct itself
-        '''
-        self.pc_time_offset = None
+        timer = ctypes.c_uint(0)
+        try:
+            kvReadTimer(self._read_handle, ctypes.byref(timer))
+        except Exception as exc:
+            # timer is usually close to 0
+            log.info(str(exc))
+        self._timestamp_offset = time.time() - (timer.value * TIMESTAMP_FACTOR)
 
         super(KvaserBus, self).__init__()
 
@@ -383,88 +396,42 @@ class KvaserBus(BusABC):
         Calling without passing any filters will reset the applied filters.
 
         Since Kvaser only supports setting one filter per handle, the filtering
-        will be done in the :meth:`recv` if more than one filter is requested.
+        will be disabled if more than one filter is requested.
 
         :param list can_filters:
-            A list of dictionaries each containing a "can_id" and a "can_mask".
+            A list of dictionaries each containing a "can_id", "can_mask" and
+            "extended".
 
-            >>> [{"can_id": 0x11, "can_mask": 0x21}]
+            >>> [{"can_id": 0x11, "can_mask": 0x21, "extended": False}]
 
             A filter matches, when ``<received_can_id> & can_mask == can_id & can_mask``
         """
-        can_id = 0
-        can_mask = 0
-
-        if not can_filters:
-            log.info('Filtering has been disabled')
-            self.sw_filters = []
-        elif len(can_filters) == 1:
+        if can_filters and len(can_filters) == 1:
             can_id = can_filters[0]['can_id']
             can_mask = can_filters[0]['can_mask']
-            log.info('canlib is filtering on ID 0x%X, mask 0x%X', can_id, can_mask)
-            self.sw_filters = []
-        elif len(can_filters) > 1:
-            log.info('Filtering is handled in Python')
-            self.sw_filters = can_filters
+            extended = 1 if can_filters[0].get('extended') else 0
+            try:
+                for handle in (self._read_handle, self._write_handle):
+                    canSetAcceptanceFilter(handle, can_id, can_mask, extended)
+            except (NotImplementedError, CANLIBError) as e:
+                log.error('Filtering is not supported - %s', e)
+            else:
+                log.info('canlib is filtering on ID 0x%X, mask 0x%X', can_id, can_mask)
 
-        # Set same filter for both handles as well as standard and extended IDs
-        for handle in (self._read_handle, self._write_handle):
-            for ext in (0, 1):
-                canSetAcceptanceFilter(handle, can_id, can_mask, ext)
+        else:
+            log.info('Hardware filtering has been disabled')
+            try:
+                for handle in (self._read_handle, self._write_handle):
+                    for extended in (0, 1):
+                        canSetAcceptanceFilter(handle, 0, 0, extended)
+            except (NotImplementedError, CANLIBError):
+                pass
 
     def flush_tx_buffer(self):
-        """
-        Flushes the transmit buffer on the Kvaser
+        """ Wipeout the transmit buffer on the Kvaser.
         """
         canIoCtl(self._write_handle, canstat.canIOCTL_FLUSH_TX_BUFFER, 0, 0)
 
-    def __convert_timestamp(self, value):
-        # The kvaser seems to zero times
-        # Use the current value if the offset has not been set yet
-        if not hasattr(self, 'timer_offset') or self.timer_offset is None:
-            self.timer_offset = value
-            self.pc_time_offset = time.time()
-
-        if value < self.timer_offset:  # Check for overflow
-            MAX_32BIT = 0xFFFFFFFF  # The maximum value that the timer reaches on a 32-bit machine
-            MAX_64BIT = 0x9FFFFFFFF  # The maximum value that the timer reaches on a 64-bit machine
-            if ctypes.sizeof(ctypes.c_long) == 8:
-                value += MAX_64BIT
-            elif ctypes.sizeof(ctypes.c_long) == 4:
-                value += MAX_32BIT
-            else:
-                raise CanError('Unknown platform. Expected a long to be 4 or 8 bytes long but it was %i bytes.' % ctypes.sizeof(ctypes.c_long))
-            if value <= self.timer_offset:
-                raise OverflowError('CAN timestamp overflowed. The timer offset was ' + str(self.timer_offset))
-
-        timestamp = float(value - self.timer_offset) / (1000000 / TIMESTAMP_RESOLUTION)  # Convert into seconds
-        timestamp += self.pc_time_offset
-        lag = time.time() - timestamp
-        if lag < 0:
-            # If we see a timestamp that is quicker than the ever before, update the offset
-            self.pc_time_offset += lag
-        return timestamp
-
-    def _is_filter_match(self, arb_id):
-        """
-        If SW filtering is used, checks if the `arb_id` matches any of
-        the filters setup.
-
-        :param int arb_id:
-            CAN ID to check against.
-
-        :return:
-            True if `arb_id` matches any filters
-            (or if SW filtering is not used).
-        """
-        if not self.sw_filters:
-            # Filtering done on HW or driver level or no filtering
-            return True
-        for can_filter in self.sw_filters:
-            if not (arb_id ^ can_filter['can_id']) & can_filter['can_mask']:
-                return True
-        return False
-
     def recv(self, timeout=None):
         """
         Read a message from kvaser device.
@@ -494,31 +461,30 @@ class KvaserBus(BusABC):
         )
 
         if status == canstat.canOK:
-            log.debug('read complete -> status OK')
-            if not self._is_filter_match(arb_id.value):
-                return None
+            #log.debug('read complete -> status OK')
             data_array = data.raw
             flags = flags.value
             is_extended = bool(flags & canstat.canMSG_EXT)
             is_remote_frame = bool(flags & canstat.canMSG_RTR)
             is_error_frame = bool(flags & canstat.canMSG_ERROR_FRAME)
-            msg_timestamp = self.__convert_timestamp(timestamp.value)
+            msg_timestamp = timestamp.value * TIMESTAMP_FACTOR
             rx_msg = Message(arbitration_id=arb_id.value,
                              data=data_array[:dlc.value],
                              dlc=dlc.value,
                              extended_id=is_extended,
                              is_error_frame=is_error_frame,
                              is_remote_frame=is_remote_frame,
-                             timestamp=msg_timestamp)
+                             channel=self.channel,
+                             timestamp=msg_timestamp + self._timestamp_offset)
             rx_msg.flags = flags
-            rx_msg.raw_timestamp = timestamp.value / (1000000.0 / TIMESTAMP_RESOLUTION)
-            log.debug('Got message: %s' % rx_msg)
+            rx_msg.raw_timestamp = msg_timestamp
+            #log.debug('Got message: %s' % rx_msg)
             return rx_msg
         else:
-            log.debug('read complete -> status not okay')
+            #log.debug('read complete -> status not okay')
             return None
 
-    def send(self, msg):
+    def send(self, msg, timeout=None):
         #log.debug("Writing a message: {}".format(msg))
         flags = canstat.canMSG_EXT if msg.id_type else canstat.canMSG_STD
         if msg.is_remote_frame:
@@ -527,12 +493,13 @@ class KvaserBus(BusABC):
             flags |= canstat.canMSG_ERROR_FRAME
         ArrayConstructor = ctypes.c_byte * msg.dlc
         buf = ArrayConstructor(*msg.data)
-        canWriteWait(self._write_handle,
-                     msg.arbitration_id,
-                     ctypes.byref(buf),
-                     msg.dlc,
-                     flags,
-                     10)
+        canWrite(self._write_handle,
+                 msg.arbitration_id,
+                 ctypes.byref(buf),
+                 msg.dlc,
+                 flags)
+        if timeout:
+            canWriteSync(self._write_handle, int(timeout * 1000))
 
     def flash(self, flash=True):
         """
@@ -550,6 +517,12 @@ class KvaserBus(BusABC):
             log.error('Could not flash LEDs (%s)', e)
 
     def shutdown(self):
+        # Wait for transmit queue to be cleared
+        try:
+            canWriteSync(self._write_handle, 100)
+        except CANLIBError as e:
+            log.warning("There may be messages in the transmit queue that could "
+                        "not be transmitted before going bus off (%s)", e)
         if not self.single_handle:
             canBusOff(self._read_handle)
             canClose(self._read_handle)
@@ -573,7 +546,7 @@ def get_channel_info(channel):
                       ctypes.byref(number), ctypes.sizeof(number))
 
     return '%s, S/N %d (#%d)' % (
-        name.value.decode(), serial.value, number.value + 1)
+        name.value.decode("ascii"), serial.value, number.value + 1)
 
 
 init_kvaser_library()
