@@ -1,8 +1,4 @@
 import threading
-try:
-    import queue
-except ImportError:
-    import Queue as queue
 
 
 class Notifier(object):
@@ -18,6 +14,8 @@ class Notifier(object):
         self.listeners = listeners
         self.bus = bus
         self.timeout = timeout
+        #: Exception raised in thread
+        self.exception = None
 
         self.running = threading.Event()
         self.running.set()
@@ -35,11 +33,15 @@ class Notifier(object):
             self._reader.join(self.timeout + 0.1)
 
     def rx_thread(self):
-        while self.running.is_set():
-            msg = self.bus.recv(self.timeout)
-            if msg is not None:
-                for callback in self.listeners:
-                    callback(msg)
-
-        for listener in self.listeners:
-            listener.stop()
\ No newline at end of file
+        try:
+            while self.running.is_set():
+                msg = self.bus.recv(self.timeout)
+                if msg is not None:
+                    for callback in self.listeners:
+                        callback(msg)
+        except Exception as exc:
+            self.exception = exc
+            raise
+        finally:
+            for listener in self.listeners:
+                listener.stop()
